% Figure2_1.m
% It requires the following functions:
% eulerE.m
% heun2.m
% rk4.m
% fPendul
% fKepler.m
% phiKepler
% fLor.m
% fABC.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initial and final time, and  Time step
%global h
t0=0;  tf=20; 

%%%%%%%%%%%%%%%  PENDULUM    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
t=[];
erEnP1=[]; erPosP1=[]; costP1=[];
erEnP2=[]; erPosP2=[]; costP2=[];
erEnP4=[]; erPosP4=[]; costP4=[];
% Initial Conditions 
x0 = [0.5 ; 0];
% Energy at initial conditions
H0=x0(2)^2/2+(1-cos(x0(1)));

for i=1:8
   t=[];   xk1=[];    xk2=[];    xk4=[]; 
   N=1*2^i;  h = (tf-t0)/N;
   [t xk1]=eulerE(@fPendul,t0,tf,x0,N);
   [t xk2]=heun2(@fPendul,t0,tf,x0,N);
   [t xk4]=rk4(@fPendul,t0,tf,x0,N);

   errorP1 = ((xk1(:,2)).^2)./2+(1-cos(xk1(:,1))) - H0;
   errorP2 = ((xk2(:,2)).^2)./2+(1-cos(xk2(:,1))) - H0;
   errorP4 = ((xk4(:,2)).^2)./2+(1-cos(xk4(:,1))) - H0;

   erEnP1=[erEnP1; max(abs(errorP1))]; costP1=[costP1; 1*N];
   erEnP2=[erEnP2; max(abs(errorP2))]; costP2=[costP2; 2*N];
   erEnP4=[erEnP4; max(abs(errorP4))]; costP4=[costP4; 4*N];
end

figure(2)
 subplot(2,2,1)
  hold on
  plot(log10(costP1),log10(erEnP1(:)),'-ok','LineWidth',1.5)
  plot(log10(costP2),log10(erEnP2(:)),'--k','LineWidth',1.5)
  plot(log10(costP4),log10(erEnP4(:)),'-k','LineWidth',1.5)
  axis([0.8 2.5 -4 1.5]) 
  xlabel('{\fontsize{12} log_{10}(N. eval.) }')   
  ylabel('{\fontsize{12} log_{10}(Err. En.)}')   
  title('{\fontsize{12} Pendulum}')   
  box

%%%%%%%%%%%%  KEPLER   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
t=[];
erEnK1=[]; erPosK1=[]; costK1=[];
erEnK2=[]; erPosK2=[]; costK2=[];
erEnK4=[]; erPosK4=[]; costK4=[];
% Initial Conditions 
e=0.2;   
x0 = [1-e ; 0; 0; sqrt((1+e)/(1-e))];
% Energy at initial conditions
H0=((x0(3))^2+(x0(4))^2)/2-1/(sqrt((x0(1))^2+(x0(2))^2));

for i=1:8
   t=[];   xk1=[];    xk2=[];    xk4=[]; 
   N=5*2^i;  h = (tf-t0)/N;
   [t xk1]=eulerE(@fKepler,t0,tf,x0,N);
   [t xk2]=heun2(@fKepler,t0,tf,x0,N);
   [t xk4]=rk4(@fKepler,t0,tf,x0,N);

   errorK1 = ((xk1(:,3)).^2+(xk1(:,4)).^2)./2-1./(sqrt((xk1(:,1)).^2+(xk1(:,2)).^2)) - H0;
   errorK2 = ((xk2(:,3)).^2+(xk2(:,4)).^2)./2-1./(sqrt((xk2(:,1)).^2+(xk2(:,2)).^2)) - H0;
   errorK4 = ((xk4(:,3)).^2+(xk4(:,4)).^2)./2-1./(sqrt((xk4(:,1)).^2+(xk4(:,2)).^2)) - H0;

    erEnK1=[erEnK1; max(abs(errorK1))]; costK1=[costK1; 1*N];
    erEnK2=[erEnK2; max(abs(errorK2))]; costK2=[costK2; 2*N];
    erEnK4=[erEnK4; max(abs(errorK4))]; costK4=[costK4; 4*N];
end

 subplot(2,2,2)
  hold on
  plot(log10(costK1),log10(erEnK1(:)),'-ok','LineWidth',1.5)
  plot(log10(costK2),log10(erEnK2(:)),'--k','LineWidth',1.5)
  plot(log10(costK4),log10(erEnK4(:)),'-k','LineWidth',1.5)
  axis([2 3 -5 0]) 
  xlabel('{\fontsize{12} log_{10}(N. eval.) }')   
  ylabel('{\fontsize{12} log_{10}(Err. En.)}')   
  title('{\fontsize{12} Kepler}')   
  box

%%%%%%%%%%%%%  LORENZ   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
t0=0;  tf=5; 
erEnP1=[]; erPosP1=[]; costP1=[];
erEnP2=[]; erPosP2=[]; costP2=[];
erEnP4=[]; erPosP4=[]; costP4=[];
% Initial Conditions 
x0 = [1 ; 2; 3];
   t=[]; x=[];
% Computing the numerical "exact" solution
  options = odeset('AbsTol',1e-10,'RelTol',1e-8);
  [t,x] = ode45('fLor',[t0,tf],x0,options);

for i=1:8
   t=[]; xk1=[];    xk2=[];    xk4=[]; 
   N=20*2^i;  h = (tf-t0)/N;
   [t xk1]=eulerE(@fLor,t0,tf,x0,N);
   [t xk2]=heun2(@fLor,t0,tf,x0,N);
   [t xk4]=rk4(@fLor,t0,tf,x0,N);

   erEnP1=[erEnP1; norm(x(end,:)-xk1(end,:))]; costP1=[costP1; 1*N];
   erEnP2=[erEnP2; norm(x(end,:)-xk2(end,:))]; costP2=[costP2; 2*N];
   erEnP4=[erEnP4; norm(x(end,:)-xk4(end,:))]; costP4=[costP4; 4*N];
end

 subplot(2,2,3)
  hold on
  plot(log10(costP1),log10(erEnP1(:)),'-ok','LineWidth',1.5)
  plot(log10(costP2),log10(erEnP2(:)),'--k','LineWidth',1.5)
  plot(log10(costP4),log10(erEnP4(:)),'-k','LineWidth',1.5)
  axis([2.2 3.7 -5 2]) 
  xlabel('{\fontsize{12} log_{10}(N. eval.) }')   
  ylabel('{\fontsize{12} log_{10}(Error)}')   
  title('{\fontsize{12} Lorenz}')   
  box


%%%%%%%%%%%%%  ABC-flow   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
t0=0;  tf=5; 
erEnP1=[]; erPosP1=[]; costP1=[];
erEnP2=[]; erPosP2=[]; costP2=[];
erEnP4=[]; erPosP4=[]; costP4=[];
% Initial Conditions 
x0 = [1 ; 2; 3];
   t=[]; x=[];
% Computing the numerical "exact" solution
  options = odeset('AbsTol',1e-10,'RelTol',1e-8);
  [t,x] = ode45('fABC',[t0,tf],x0,options);

for i=1:9
   t=[]; xk1=[];    xk2=[];    xk4=[]; 
   N=2*2^i;  h = (tf-t0)/N;
   [t xk1]=eulerE(@fABC,t0,tf,x0,N);
   [t xk2]=heun2(@fABC,t0,tf,x0,N);
   [t xk4]=rk4(@fABC,t0,tf,x0,N);

   erEnP1=[erEnP1; norm(x(end,:)-xk1(end,:))]; costP1=[costP1; 1*N];
   erEnP2=[erEnP2; norm(x(end,:)-xk2(end,:))]; costP2=[costP2; 2*N];
   erEnP4=[erEnP4; norm(x(end,:)-xk4(end,:))]; costP4=[costP4; 4*N];
end

 subplot(2,2,4)
  hold on
  plot(log10(costP1),log10(erEnP1(:)),'-ok','LineWidth',1.5)
  plot(log10(costP2),log10(erEnP2(:)),'--k','LineWidth',1.5)
  plot(log10(costP4),log10(erEnP4(:)),'-k','LineWidth',1.5)
  axis([1 3 -5 1.5]) 
  xlabel('{\fontsize{12} log_{10}(N. eval.) }')   
  ylabel('{\fontsize{12} log_{10}(Error)}')   
  title('{\fontsize{12} ABC-flow}')   
  box
